################################################################################
# DA + FS
################################################################################

options(stringsAsFactors = F)
library(pROC)
library(glmnet)
Seed = 2022
set.seed(Seed)


alpha = 0.9
supervised_DA = TRUE #TRUE or FALSE
input_path = "D:/YC/data/processed/"
L_super = c("Unsupervised","Supervised")
output_path = paste0("D:/YC/data/preprocessing/DA/R/",L_super[supervised_DA+1],"/alpha=",alpha,"/")
### --- ###
i = 2
### --- ###
#qsub -l nodes=node05 -q hmque Experiment_R02_DA_FS_LogRidge_revised_220719.sh

input_name.list = c(
  "2014_Docetaxel",
  "2014_Erlotinib",
  "2014_Sorafenib",
  "PDX_Erlotinib",
  "PDX_Cetuximab",
  "PDX_Gemcitabine",
  "PDX_Paclitaxel",
  "TCGA_Docetaxel",
  "TCGA_Cisplatin",
  "TCGA_Gemcitabine"
)

## paras
range_cut_off = c(850, 370, 403, 370, 877, 866, 399, 850, 850, 866)
train_IC50_cut_off = c(0.0125, 2, 4, 2, 65.8, 1.024, 0.1024, 0.0125, 10, 1.024)





# LogRidge_kfold_searchLamb_train = source("LogRidge_kfold_searchLambda.R")
# ks_DomainAdaptor = source("KS_DomainAdpator.R")
# FDR = source("DE_test_FDR.R")
# BW_ratio = source("BW_ratio.R")

FDR <- function(GED_GDSC.df){
  
  answer <- GED_GDSC.df$IC50
  sen <- which(answer == 1)
  res <- which(answer == 0)
  tumors = c(sen,res)
  ##this for test sets
  #GED_GDSC.df.new <- GED_GDSC.df[which(rownames(GED_GDSC.df)%in%names(tumors)),]
  #GED_GDSC.df.new <- GED_GDSC.df.new[match(names(tumors),rownames(GED_GDSC.df.new)),]
  
  GED_GDSC.df.new <- t(GED_GDSC.df)
  
  # t.test
  GAPDH = which(rownames(GED_GDSC.df.new)=='GAPDH')
  IC50 = which(rownames(GED_GDSC.df.new)=='IC50')
  GED_GDSC.df.new <- GED_GDSC.df.new[c(-IC50,-GAPDH),]
  x = GED_GDSC.df.new[, sen]
  y = GED_GDSC.df.new[, res]
  
  ttest <- matrix(NA,nrow(GED_GDSC.df.new),4)
  colnames(ttest) <- c("Ftest","sig.","statistic","p.value")
  for(i in 1:nrow(GED_GDSC.df.new)){
    #cat(i,"\n")
    ttest[i,1] <- var.test(x[i,], y[i,], ratio = 1, alternative = "two.sided")$p.value
    ttest[i,2] <- (ttest[i,1] < 0.05) #var.equal = F
    
    if(ttest[i,2]==T){
      ttest[i,3] <- t.test(x[i,],y[i,],var.equal = F)$statistic
      ttest[i,4] <- t.test(x[i,],y[i,],var.equal = F)$p.value
    }else if(ttest[i,2]==F){
      ttest[i,3] <- t.test(x[i,],y[i,],var.equal = T)$statistic
      ttest[i,4] <- t.test(x[i,],y[i,],var.equal = T)$p.value
    }else{}
    
    #t.test(x[i,],y[i,])$p.value
    rownames(ttest) <- colnames(GED_GDSC.df)[c(-IC50,-GAPDH)]
    
  }
  ###output fdr csv
  FDR <- p.adjust(ttest[,4], method ="BH", n = nrow(GED_GDSC.df.new))
  GDSC_ttest <- round(cbind(ttest,FDR),5)
  GDSC_ttest_order <- GDSC_ttest[order(GDSC_ttest[,4],decreasing = F),]
  return(GDSC_ttest_order)
}

BW_ratio <- function(GDSC.fdr){
  
  sen_GDSC <- GDSC.fdr[GDSC.fdr$IC50 == 1,-1]
  sen_count <- nrow(sen_GDSC )
  res_GDSC <- GDSC.fdr[GDSC.fdr$IC50 == 0,-1]
  res_count <- nrow(res_GDSC )
  BW = c()
  
  for(i in 2:ncol(GDSC.fdr)){
    x_1j <- as.array(sen_GDSC[,i-1])
    x_0j <- as.array(res_GDSC[,i-1])
    x_j <- as.array(GDSC.fdr[,i])
    bss <- sen_count*(mean(x_1j)-mean(x_j))^2+res_count*(mean(x_0j)-mean(x_j))^2
    wss <- sum((x_1j-mean(x_1j))^2)+sum((x_0j-mean(x_0j))^2)
    BW <- c(BW, bss/wss)
  }
  
  bw_ratio_list <- colnames(GDSC.fdr[,-1])[order(BW, decreasing = TRUE)]
}

ks_DomainAdaptor <- function(X_source, X_target, alpha, supervised_DA, y_source=NULL, y_target=NULL) {
  print("Domain adaption starts")
  
  n_genes = ncol(X_source)
  s_pval = numeric(n_genes)
  r_pval = numeric(n_genes)
  
  if (supervised_DA) {
    
    for(i in 1:n_genes) {
      s1 = X_source[(y_source == 1), i]
      r1 = X_source[(y_source == 0), i]
      
      s2 = X_target[(y_target == 1), i]
      r2 = X_target[(y_target == 0), i]
      
      s_pval[i] = ks.test(s1, s2)$p.value
      r_pval[i] = ks.test(r1, r2)$p.value
      
    }
    
    set1_features_s = colnames(X_source)[(s_pval >= alpha)]
    set1_features_r = colnames(X_source)[(r_pval >= alpha)]
    set1_features = intersect(set1_features_s, set1_features_r)
    
  }
  else {
    
    for(i in 1:n_genes) {
      s1 = X_source[, i]
      s2 = X_target[, i]
      
      s_pval[i] = ks.test(s1, s2)$p.value
    }
    
    set1_features = colnames(X_source)[s_pval >= alpha]
    
  }
  
  return(set1_features)
}



input_name = unlist(strsplit(input_name.list[i], "_"))
target_type = input_name[1]
drug_name = input_name[2]
n_CLs = range_cut_off[i]
IC50_cut_off = train_IC50_cut_off[i]


data <- read.csv(file=paste0(input_path, "combat_GDSC_", target_type, "_", drug_name, ".csv"), sep=',')
n_data = nrow(data)

GDSC_response = data[1:n_CLs, ]
row.names(GDSC_response) <- GDSC_response[, 1]; GDSC_response <- GDSC_response[, -1]
response_IC50 = GDSC_response$IC50
response_IC50 = ifelse(response_IC50<=IC50_cut_off, 1, 0)

Clinical_response <- data[(1 + n_CLs):n_data, ]
Clinical_response <- Clinical_response[, -1]
colnames(Clinical_response) <- c("IC50", colnames(Clinical_response)[-1])


## Domain adaption (KS test)
KS_list = ks_DomainAdaptor(GDSC_response[, -1], Clinical_response[, -1], alpha=alpha, supervised_DA=supervised_DA, response_IC50, Clinical_response)
GDSC_response_sel_DA = GDSC_response[c("IC50", KS_list)]

## FDR
GDSC_response_sel_DA_binary = GDSC_response_sel_DA
GDSC_response_sel_DA_binary$IC50 = response_IC50
fdr_table <- FDR(GDSC_response_sel_DA_binary)
GDSC_fdr_DA <- GDSC_response_sel_DA_binary[, c("IC50", row.names(fdr_table))]
GDSC_colnames_length = length(intersect(colnames(GDSC_fdr_DA), colnames(Clinical_response)))
GDSC_fdr_DA <- GDSC_fdr_DA[, intersect(colnames(GDSC_fdr_DA), colnames(Clinical_response))[1:(min(GDSC_colnames_length,1001))]]
Clinical_fdr_DA <- Clinical_response[colnames(GDSC_fdr_DA)]

## BW-ratio
bw_ratio_list_DA <- BW_ratio(GDSC_fdr_DA)
GDSC_bwr_DA <- GDSC_fdr_DA[, c("IC50", bw_ratio_list_DA)]
Clinical_bwr_DA <- Clinical_fdr_DA[, c("IC50", bw_ratio_list_DA)]



## CSV save
save_data <- cbind(rownames(GDSC_bwr_DA),GDSC_bwr_DA[-1])
colnames(save_data) <- c('',colnames(GDSC_bwr_DA[-1]))

write.table(
  save_data,
  paste0(
    output_path,
    '/',
    target_type,
    '_',
    drug_name,
    "(top 1000 genes).csv"
  ),
  row.names = F,
  sep = ","
)









