# %%
'''
2021-04-13
KNN 5-fold CV(10 times)
Input: combat_GDSC_{}_{}.csv, {}_{}(top 1000 genes).csv
Output: KNN(5-fold)_report_{}_{}.csv
'''

# %%
# scikit-learn  0.21.3
from sklearn.linear_model import RidgeClassifier
from sklearn.linear_model import LogisticRegression, Perceptron
from sklearn.model_selection import GridSearchCV, train_test_split
from sklearn.neighbors import KNeighborsClassifier
from sklearn.pipeline import Pipeline
from sklearn.svm import SVC

import pandas as pd
import numpy as np
import os
import matplotlib.pyplot as plt
import time
import math

from sklearn.ensemble import RandomForestClassifier
from sklearn.feature_selection import SelectFromModel
from sklearn.model_selection import train_test_split
from sklearn.metrics import auc, confusion_matrix,  f1_score, precision_score, recall_score, roc_curve,roc_auc_score, average_precision_score
from sklearn.model_selection import cross_val_score
from sklearn.model_selection import StratifiedKFold,RepeatedStratifiedKFold
from scipy.stats import spearmanr
from scipy.stats import ks_2samp
from scipy import stats

#import rpy2
#from rpy2.robjects.packages import importr
#from rpy2.robjects.vectors import FloatVector

# %%
import warnings
warnings.filterwarnings("ignore")
#os.chdir('C:/Users/gshieh/Desktop/YC')

input_path = 'D:/YC/data/processed/'

# %%
# Type    Drug           CL     cut
# 2014    Docetaxel      850    0.0125
# 2014    Erlotinib      370    2
# 2014    Sorafenib      403    4
# PDX     Cetuximab      877    65.8
# PDX     Erlotinib      370    2
# PDX     Gemcitabine    866    1.024
# PDX     Paclitaxel     399    0.1024
# TCGA    Cisplatin      850    10
# TCGA    Docetaxel      850    0.0125
# TCGA    Gemcitabine    866    1.024
Type_Drug = [ 
             ['2014','Docetaxel',850,0.0125], #0 , 2014_Docetaxel
             ['2014','Erlotinib',370,2], #1 , 2014_Erlotinib
             ['2014','Sorafenib',403,4], #2 , 2014_Sorafenib
             ['PDX','Cetuximab',877,65.8], #3 , PDX_Cetuximab
             ['PDX','Erlotinib',370,2], #4 , PDX_Erlotinib
             ['PDX','Gemcitabine',866,1.024], #5 , PDX_Gemcitabine
             ['PDX','Paclitaxel',399,0.1024], #6 , PDX_Paclitaxel
             ['TCGA','Cisplatin',850,10],  #7 , TCGA_Cisplatin
             ['TCGA','Docetaxel',850,0.0125],  #8 , TCGA_Docetaxel
             ['TCGA','Gemcitabine',866,1.024]  #9 , TCGA_Gemcitabine
]


data_type,drug,CL,cut = Type_Drug[4]
supervised = True # True: supervised , False : unsupervised
withR = True # True: DA with R , False : DA with python
Alpha = 0.7
n_rep = 10
n_jobs = -1
#qsub -l nodes=node61 -q hmque KNN_5-fold_cv.sh
#qsub -l nodes=node61 -q hmque KNN_5-fold_cvtest.sh

#num_DEgene = np.arange(10, 1010, 10) # 50, 60, 70, ..., 1000
def list_generate(num_End): #50(10)200(20)400(100)1000
    End_num = np.array([50,200,400,1000])
    K = [10,20,100]
    if num_End < 1000:
        Index = np.sum(End_num < num_End)
        if Index == 0 :
            return [num_End]
        else:
            List = []
            for i in range(Index):
                List += list(np.arange(End_num[i],min(End_num[i+1],num_End),K[i]))
            List += [num_End]
        return List
    else:
        return list(np.arange(50,200,10))+list(np.arange(200,400,20))+list(np.arange(400,1000,100)) + [1000]
#num_DEgene = np.array(list_generate(1000))

# %%
# Import dataset
superset = {True:'Supervised' , False : 'Unsupervised'}
withRset = {True:'R' , False:'python'}
input_drug_path = 'D:/YC/data//preprocessing/DA/{}/{}/alpha={}'.format(withRset[withR],superset[supervised],Alpha)
#/NA3/gshieh2/YC/Cluster/1000/data/output/KNN/DA_Supervised/alpha=0.9
output_path = 'D:/YC/data//output/KNN/DA_{}/{}/alpha={}'.format(withRset[withR],superset[supervised],Alpha)
startTime1 = time.ctime()
startTime2 = time.time()
print('Start time: ', startTime1)
print('Read data: {}_{}'.format(data_type, drug))
print('-' * 100)

data = pd.read_csv('{}/combat_GDSC_{}_{}.csv'.format(input_path, data_type, drug), header = 0, index_col = 0)
Erlotinib_GED_Ttest = data.iloc[:CL, :]
Erlotinib_GED_Ttest['IC50'] = np.where(Erlotinib_GED_Ttest['IC50'] < cut, 1, 0)
Clinical_GED_Ttest = data.iloc[CL:, :]

# %%
# Top 1000 genes
Drug = pd.read_csv('{}/{}_{}(top 1000 genes).csv'.format(input_drug_path, data_type, drug), \
                   header = 0, index_col = 0)
Gene_1000 = Drug.columns
num_DEgene = list_generate(len(Gene_1000))
# %%
# Model training
from sklearn.neighbors import KNeighborsClassifier
print('Train&test')
print('-' * 100)
num_p = []
cv_result = []
features = []
train_roc_auc = []
train_prc_auc = []
test_roc_auc = []
test_prc_auc = []
best_k = []
param_k = []
run_time = []
sqrt_n = int(np.floor(np.sqrt(Erlotinib_GED_Ttest.shape[0])))

# %%
# Select k
for k in reversed(range(sqrt_n + 1)):
    if k % 2 != 0:
        param_k.append(k)
    if len(param_k) == 7:
        break

# %%
for i in num_DEgene:
    print('\np = {}'.format(i))
    num_p.append(i)
    feature = Gene_1000[:i]
    X_train = Erlotinib_GED_Ttest[feature]
    y_train = Erlotinib_GED_Ttest['IC50'].values

    X_test = Clinical_GED_Ttest[feature]
    y_test = Clinical_GED_Ttest['IC50'].values

    ##### calculate AUC
    # set the param_grid parameter of GridSearchCV to a list of dictionaries
    def mydist(X, y):
        return 1 - spearmanr(X, y)[0]
    knn = KNeighborsClassifier(metric = mydist)
    
    kfold = RepeatedStratifiedKFold(n_splits = 5, random_state = 0, n_repeats = n_rep)
    param_grid = [{'n_neighbors': param_k}]
    gs = GridSearchCV(estimator = knn, 
                      param_grid = param_grid, 
                      scoring = 'roc_auc', 
                      cv = kfold, 
                      n_jobs = n_jobs) #
    # Parameter tuning
    tStart = time.time()
    gs = gs.fit(X_train, y_train)
    tEnd = time.time()

    run_time.append(tEnd - tStart)
    cv_result.append(gs.best_score_)
    best_k.append(gs.best_params_.get('n_neighbors'))
    print('It costs {:.2f} sec.'.format(tEnd - tStart))
    print('Best score = {}'.format(gs.best_score_))
    print('Best parameter = {}'.format(gs.best_params_))

    # Fit model with the best parameter
    clf = gs.best_estimator_
    clf.fit(X_train, y_train)
    
    ## Train set
    y_train_score = clf.predict_proba(X_train.values)[:, 1]
    train_roc_auc.append(roc_auc_score(y_train, y_train_score))
    train_prc_auc.append(average_precision_score(y_train, y_train_score))
    print('Train ROC_AUC = {}'.format(roc_auc_score(y_train, y_train_score)))
    print('Train AP(Average Precision) = {}'.format(average_precision_score(y_train, y_train_score)))

    ## Test set
    y_test_score = clf.predict_proba(X_test.values)[:, 1]
    test_roc_auc.append(roc_auc_score(y_test, y_test_score))
    test_prc_auc.append(average_precision_score(y_test, y_test_score))
    print('Test ROC_AUC = {}'.format(roc_auc_score(y_test, y_test_score)))
    print('Test AP(Average Precision) = {}'.format(average_precision_score(y_test, y_test_score)))

# %%
num_p = np.asarray(num_p)
num_p = num_p.reshape(len(num_p), 1)
num_p = pd.DataFrame(num_p)
num_p.index = ['p={}'.format(i) for i in num_DEgene[:len(num_p)]]
num_p.columns = ['Num_p']

best_k = np.asarray(best_k)
best_k = best_k.reshape(len(best_k), 1)
best_k = pd.DataFrame(best_k)
best_k.index = ['p={}'.format(i) for i in num_DEgene[:len(best_k)]]
best_k.columns = ['Best_k']

cv_result = np.asarray(cv_result)
cv_result = cv_result.reshape(len(cv_result), 1)
cv_result = pd.DataFrame(cv_result)
cv_result.index = ['p={}'.format(i) for i in num_DEgene[:len(cv_result)]]
cv_result.columns = ['CV_score']

## Train set
train_roc_auc = np.asarray(train_roc_auc)
train_roc_auc = train_roc_auc.reshape(len(train_roc_auc), 1)
train_roc_auc = pd.DataFrame(train_roc_auc)
train_roc_auc.index = ['p={}'.format(i) for i in num_DEgene[:len(train_roc_auc)]]
train_roc_auc.columns = ['Train_roc_auc']

train_prc_auc = np.asarray(train_prc_auc)
train_prc_auc = train_prc_auc.reshape(len(train_prc_auc), 1)
train_prc_auc = pd.DataFrame(train_prc_auc)
train_prc_auc.index = ['p={}'.format(i) for i in num_DEgene[:len(train_prc_auc)]]
train_prc_auc.columns = ['Train_prc_auc']

## Test set
test_roc_auc = np.asarray(test_roc_auc)
test_roc_auc = test_roc_auc.reshape(len(test_roc_auc), 1)
test_roc_auc = pd.DataFrame(test_roc_auc)
test_roc_auc.index = ['p={}'.format(i) for i in num_DEgene[:len(test_roc_auc)]]
test_roc_auc.columns = ['Test_roc_auc']

test_prc_auc = np.asarray(test_prc_auc)
test_prc_auc = test_prc_auc.reshape(len(test_prc_auc), 1)
test_prc_auc = pd.DataFrame(test_prc_auc)
test_prc_auc.index = ['p={}'.format(i) for i in num_DEgene[:len(test_prc_auc)]]
test_prc_auc.columns = ['Test_prc_auc']

## Run time
run_time = np.asarray(run_time)
run_time = run_time.reshape(len(run_time), 1)
run_time = pd.DataFrame(run_time)
run_time.index = ['p={}'.format(i) for i in num_DEgene[:len(run_time)]]
run_time.columns = ['Run_time(5-fold)']

# %%
result = pd.concat([num_p, best_k, cv_result, 
                   train_roc_auc, train_prc_auc, 
                   test_roc_auc, test_prc_auc, 
                   run_time], axis = 1)


# %%
try:
    result.to_csv('{}/KNN(5-fold)_report_{}_{}.csv'.format(output_path, data_type, drug))
except:
    os.mkdir(str(output_path))
    result.to_csv('{}/KNN(5-fold)_report_{}_{}.csv'.format(output_path, data_type, drug))
# %%
endTime1 = time.ctime()
endTime2 = time.time()
print('\nEnd time: ', endTime1)
print('Total cost time:', endTime2 - startTime2)

# %%

