
rm(list=ls())

options(stringsAsFactors = F)
library(pROC)
library(glmnet)
Seed = 2022
set.seed(Seed)

### --- ###
alpha = 0.7
i = 1
### --- ###
#qsub -l nodes=node05 -q hmque Experiment_R02_DA_FS_LogRidge_revised_220719.sh


input_name.list = c(
    "2014_Docetaxel", #1
    "2014_Erlotinib", #2
    "2014_Sorafenib", #3 
    "PDX_Erlotinib",  #4
    "PDX_Cetuximab",  #5
    "PDX_Gemcitabine",#6
    "PDX_Paclitaxel", #7
    "TCGA_Docetaxel", #8
    "TCGA_Cisplatin", #9
    "TCGA_Gemcitabine"#10
)

supervised_DA = TRUE #TRUE or FALSE
input_path = "D:/YC/data/processed/"
input_path2 = paste0("D:/YC/data/preprocessing/DA/R/Supervised/alpha=",alpha,'/')
L_super = c("Unsupervised","Supervised")
output_path = paste0("D:/YC/data/output/LogRidge/DA_",L_super[supervised_DA+1],"/alpha=",alpha,"_seed/")
output_model = paste0("D:/YC/data/output/LogRidge/DA_",L_super[supervised_DA+1],"/alpha=",alpha,"_seed/model/")


## paras
range_cut_off = c(850, 370, 403, 370, 877, 866, 399, 850, 850, 866)
train_IC50_cut_off = c(0.0125, 2, 4, 2, 65.8, 1.024, 0.1024, 0.0125, 10, 1.024)




LogRidge_kfold_searchLamb_train = function(
    inputs.train,
    inputs.test,
    drug_GDSC,
    target_type,
    output_path,
    output_model,
    gene_vector,
    seed_list,
    seed_i,
    train_cut_off=NULL) {
    
    ## default parameters
    alpha = 0
    fold_id = NULL
    nfolds = 5
    type.measure = "auc"
    stratified = T
    ##
    ### ----------------------------------------------- ###
    
    train.x = inputs.train[[1]]
    train.y = inputs.train[[2]]
    
    train.y_log <- train.y
    if (sum(train.y_log %in% 0:1) != length(train.y_log))
        train.y_log <- ifelse(train.y_log <= train_cut_off, 1, 0)
    
    test.x = inputs.test[[1]]
    test.y = inputs.test[[2]]
    test.y_log = unlist(test.y)
    
    
    CLGD = cbind(c('(Intercept)',colnames(data.frame(inputs.train[1]))))
    colnames(CLGD) <- 'gene'
    
    # type.measure = match.arg(type.measure)
    
    # if (!is.null(lambda) && length(lambda) < 2)
    #     stop("Stop: need more than one value of lambda for CV")
    
    n = nrow(train.x)
    # if (is.null(fold_id)) {
    #     if (stratified) {
    #         set.seed(seeds[1])
    #         fold0 <- sample.int(sum(train.y_log == 0)) %% nfolds
    #         set.seed(seeds[2])
    #         fold1 <- sample.int(sum(train.y_log == 1)) %% nfolds
    #         fold_id <- numeric(length(train.y_log))
    #         fold_id[train.y_log == 0] <- fold0
    #         fold_id[train.y_log == 1] <- fold1
    #         fold_id <- fold_id + 1
    #         
    #     }
    #     else {
    #         set.seed(seeds[i, 1])
    #         fold_id = sample(rep(seq(nfolds), length = n))
    #     }
    # }
    
    ### training ====================================================
    ## cross validations
    n_rep = 10
    train_df <- data.frame()
    GDSC_df <- data.frame()
    Clinical_df <- data.frame()
    #gene_vector = c(seq(50, 200, 10), seq(220, 500, 20), seq(550, 1000, 50))
    
    print(paste0(nfolds, " cv start"))
    
    for (gene in gene_vector) {
        print(gene)
        GDSC_y <- train.y_log
        GDSC_X <- train.x[, 1:gene]
        Clinical_X <- test.x[, 1:gene]
        Clinical_y <- test.y_log
        Clinical_binary <- as.matrix(Clinical_y)
        
        # --------- TRAINING -----------
        
        
        auc_list <- c()
        
        #lambda
        s = seq(0, -3, length = 10)
        
        for (ite in 1:2) {
            grid = 10 ^ s
            auc_list_ite <- c()
            se_list_ite <- c()
            
            ### -------------- Lambda tuning starts -------------- ###
            print(paste0(
                "----------- Iteration ",
                ite,
                " Lambda tuning starts -----------"
            ))
            
            #### ------- Hyper-parameters tuning ------ ####
            for (lambda in grid) {
                # cv.logridge.pred.class <- rep(0, nrow(GDSC_X))
                # cv.logridge.pred.response <- rep(0, nrow(GDSC_X))
                
                order = sample(1:nrow(GDSC_X), nrow(GDSC_X), replace = FALSE)
                
                ### -------------- CV starts -------------- ###
                #print(paste0(
                #    "----------- lambda = ",
                #    lambda,
                #    ", CV starts -----------"
                #))
                
                auc_list_rep = numeric(n_rep)
                ### ------- Repetition for CV ------ ###
                set.seed(2022)
                for (rep in 1:n_rep) {
                    
                    auc_logridge_fold = numeric(nfolds)
                    

                    if (stratified) {
                        # set.seed(seeds[1])
                        fold0 <- sample.int(sum(train.y_log == 0)) %% nfolds
                        # set.seed(seeds[2])
                        fold1 <- sample.int(sum(train.y_log == 1)) %% nfolds
                        fold_id <- numeric(length(train.y_log))
                        fold_id[train.y_log == 0] <- fold0
                        fold_id[train.y_log == 1] <- fold1
                        fold_id <- fold_id + 1
                    }
                    else {
                        # set.seed(seeds[i, 1])
                        fold_id = sample(rep(seq(nfolds), length = n))
                    }

                    
                    for (j in 1:nfolds) {
                        # Exclude them from the train set
                        train.x_cv <- GDSC_X[fold_id != j,]
                        train.y_cv <- GDSC_y[fold_id != j]
                        # Include them in the test set
                        val.x <- GDSC_X[fold_id == j,]
                        
                        
                        logridge.mod <-
                            glmnet(
                                as.matrix(train.x_cv),
                                as.matrix(train.y_cv),
                                alpha = 0,
                                family = "binomial",
                                lambda = lambda
                                
                            )
                        
                        # ------ for testing insignificant celllines --------
                        # cv.logridge.pred.response[j] <- predict(logridge.mod, s = lambda, newx = as.matrix(test.x), family="binomial", type = "response", type.measure = "auc")
                        
                        cv.logridge.pred.class <-
                            predict(
                                logridge.mod,
                                s = lambda,
                                newx = as.matrix(val.x),
                                family = "binomial",
                                type = "class"
                            )
                        cv.logridge.pred.response <-
                            predict(
                                logridge.mod,
                                s = lambda,
                                newx = as.matrix(val.x),
                                family = "binomial",
                                type = "response"
                            )
                        
                        roc_fold <- roc(
                            actual ~ pred,
                            data = data.frame(
                                actual = GDSC_y[fold_id == j],
                                pred = as.numeric(cv.logridge.pred.response)
                            ),
                            levels = c(0, 1),
                            direction = "<"
                        )
                        # optimal_cut_off_fold <-
                        #     coords(roc_fold, "best", transpose = FALSE)$threshold
                        
                        auc_logridge_fold[j] = roc_fold$auc
                    } # End of CV in each rep.
                    
                    # auc_list_rep <- c(auc_list_rep, mean(auc_logridge_fold)) # length == n_rep
                    auc_list_rep[rep] = mean(auc_logridge_fold)
                    
                } ## End for repetition of CV
                
                #print(paste0(rep, " repeats end: mean AUC(val) => ", mean(auc_list_rep)))
                auc_list_ite <- c(auc_list_ite, mean(auc_list_rep))
                se_list_ite <- c(se_list_ite, sd(auc_list_rep))
                #print(se_list_ite)
            }
            ### -------------- Lambda tuning ends -------------- ###
            
            argmax_auc = which.max(auc_list_ite) # if the current AUC -- highest, replace the argmax_auc => Find the corresponding hyper-parameter (lambda)
            bestlam <- grid[argmax_auc]
            AUC <- auc_list_ite[argmax_auc]
            SE <- se_list_ite[argmax_auc]
            #print(SE)
            #cat("ite=",ite,argmax_auc,s[argmax_auc],"\n")
            s = seq(s[argmax_auc] + 0.5 / (10 ^ ite), s[argmax_auc] - 0.5 /
                        (10 ^ ite), length = 10)
        }
        
        #LOOCV result
        print('LOOCV')
        GDSC_df <-
            rbind(GDSC_df, c(gene, round(bestlam, 4), round(AUC, 3), round(SE,4)))
        
        #---------------------TESTING--------------------------------
        print("TESTING")
        logridge.mod <-
            glmnet(
                as.matrix(GDSC_X),
                GDSC_y,
                alpha = 0,
                lambda = bestlam,
                family = "binomial",
                
            )
        
        
        #test
        print("TEST")
        cv.logridge.pred.response <-
            predict(
                logridge.mod,
                s = bestlam,
                newx = as.matrix(Clinical_X),
                family = "binomial",
                type = "response",
                
            )
        
        
        #AUC
        print("AUC")
        roc_list <- roc(
            actual ~ pred,
            data = data.frame(
                actual = Clinical_y,
                pred = as.numeric(cv.logridge.pred.response)
            ),
            levels = c(0, 1),
            direction = "<"
        )
        
        save(logridge.mod, file=paste0(output_model,target_type,"_",drug_GDSC,"_p=",gene,"_seed_i=",seed_i,".RData"))
        
        DFCD = data.frame(cbind(rownames(coef(logridge.mod))), coef(logridge.mod)[,1])
        names(DFCD) <- c('gene',paste0('p = ',gene))
        CLGD = merge(CLGD,DFCD,by = 'gene', all = T, sort = F)
        # optimal_cut_off <-
        #     coords(roc_list, "best", transpose = FALSE)$threshold
        
        auc_logridge = roc_list$auc
        
        optimal_cut_off = 0.5
        
        #Accuracy
        print("Accuracy")
        tp <-
            sum((cv.logridge.pred.response >= optimal_cut_off) * (Clinical_y == 1))
        tn <-
            sum((cv.logridge.pred.response < optimal_cut_off) * (Clinical_y == 0))
        tpr <- tp / sum(Clinical_y == 1)
        fpr <-  (sum(Clinical_y == 0) - tn) / sum(Clinical_y == 0)
        
        accuracy <- round((tn + tp) / nrow(Clinical_y), 2)
        Clinical_binary <-
            cbind(Clinical_binary,
                  ifelse(cv.logridge.pred.response > optimal_cut_off, 1, 0))
        Clinical_df <-
            rbind(Clinical_df,
                  c(gene, bestlam, tp, tn, tpr, fpr, auc_logridge))
    }
    print("write.table")
    colnames(GDSC_df) <- c("gene", "lambda", "AUC", "SE")
    colnames(Clinical_df) <-
        c("gene", "lambda", "TP", "TN", "TPR", "FPR", "AUC")
    
    write.table(
        GDSC_df,
        paste0(
            output_path,
            target_type,
            "_",
            drug_GDSC,
            "_seed_i=",seed_i,
            "_5foldCV_LogicRidge_FDR_Top1000_BW.csv"
        ),
        row.names = F,
        sep = ","
    )
    write.table(
        Clinical_df,
        paste0(
            output_path,
            "Clinical_",
            target_type,
            "_",
            drug_GDSC,
            "_seed_i=",seed_i,
            "_5foldCV_LogicRidge_FDR_Top1000_BW.csv"
        ),
        row.names = F,
        sep = ","
    )
    write.table(
        Clinical_binary,
        paste0(
            output_path,
            "Clinical_",
            target_type,
            "_",
            drug_GDSC,
            "_seed_i=",seed_i,
            "_5foldCV_LogicRidge_FDR_Top1000_BW_Binary.csv"
        ),
        row.names = F,
        sep = ","
    )
    write.table(
        CLGD,
        paste0(
            output_path,
            "Clinical_",
            target_type,
            "_",
            drug_GDSC,
            "_seed_i=",seed_i,
            "_5foldCV_LogicRidge_FDR_Top1000_BW_beta_list.csv"
        ),
        row.names = F,
        sep = ","
    )
}


num.gene_vec.f <- function(x){
    if (x>=1000){
        return(c(seq(50, 200, 10), seq(220, 500, 20), seq(550, 1000, 50)))
    }else if(x<=200){
        return(c(seq(50,x-1,10),x))
    }else if(x<=500){
        return(c(seq(50, 199, 10), seq(200, x-1, 20),x))
    }else{
        return(c(seq(50, 200, 10), seq(220, 499, 20), seq(500, x-1, 50),x))
        }
    }





input_name = unlist(strsplit(input_name.list[i], "_"))
target_type = input_name[1]
drug_name = input_name[2]
n_CLs = range_cut_off[i]
IC50_cut_off = train_IC50_cut_off[i]


data <- read.csv(file=paste0(input_path, "combat_GDSC_", target_type, "_", drug_name, ".csv"), sep=',')
n_data = nrow(data)

GDSC_response = data[1:n_CLs, ]
row.names(GDSC_response) <- GDSC_response[, 1]; GDSC_response <- GDSC_response[, -1]
response_IC50 = GDSC_response$IC50
response_IC50 = ifelse(response_IC50<=IC50_cut_off, 1, 0)

Clinical_response <- data[(1 + n_CLs):n_data, ]
Clinical_response <- Clinical_response[, -1]
colnames(Clinical_response) <- c("IC50", colnames(Clinical_response)[-1])

data2 <- read.csv(file=paste0(input_path2, target_type, "_", drug_name, "(top 1000 genes).csv"), sep=',')
pre_data = data.frame(data2[,-1])
rownames(pre_data) = data2[,1]


gene_vector = num.gene_vec.f(ncol(pre_data))

LogRidge_kfold_searchLamb_train(
    inputs.train = list(pre_data, response_IC50),
    inputs.test = list(Clinical_response[colnames(pre_data)], Clinical_response$IC50),
    drug_GDSC = drug_name,
    target_type = target_type,
    output_path = output_path,
    output_model = output_model,
    gene_vector = gene_vector,
    seed_list = seed_list,
    seed_i = seed_i,
    train_cut_off = IC50_cut_off
)
